@@ -1,7 +1,6 @@
 //! [Church pair](https://en.wikipedia.org/wiki/Church_encoding#Church_pairs)
 
 use term::*;
-use term::Term::*;
 use term::Error::*;
 use booleans::*;
 
@@ -43,7 +42,7 @@ pub fn pair() -> Term {
 ///
 /// let pair_0_1 = app!(pair(), zero(), one());
 ///
-/// assert_eq!(beta(fst().app(pair_0_1), NOR, 0), zero());
+/// assert_eq!(beta(fst().app(pair_0_1), NOR, 0, false), zero());
 /// # }
 /// ```
 pub fn fst() -> Term { abs(Var(1).app(tru())) }
@@ -63,7 +62,7 @@ pub fn fst() -> Term { abs(Var(1).app(tru())) }
 ///
 /// let pair_0_1 = app!(pair(), zero(), one());
 ///
-/// assert_eq!(beta(snd().app(pair_0_1), NOR, 0), one());
+/// assert_eq!(beta(snd().app(pair_0_1), NOR, 0, false), one());
 /// # }
 /// ```
 pub fn snd() -> Term { abs(Var(1).app(fls())) }
@@ -276,7 +275,7 @@ impl From<(Term, Term)> for Term {
 }
 
 #[cfg(test)]
-mod test {
+mod tests {
     use super::*;
     use reduction::beta;
     use reduction::Order::*;
@@ -284,12 +283,12 @@ mod test {
     #[test]
     fn pair_from_pair() {
         assert_eq!(Term::from((0.into(), 1.into())),
-                   beta(app!(pair(), 0.into(), 1.into()), NOR, 0));
+                   beta(app!(pair(), 0.into(), 1.into()), NOR, 0, false));
     }
 
     #[test]
     fn pair_operations() {
-        let pair_four_three = beta(app!(pair(), 4.into(), 3.into()), NOR, 0);
+        let pair_four_three = beta(app!(pair(), 4.into(), 3.into()), NOR, 0, false);
 
         assert!(pair_four_three.is_pair());
 
