@@ -1,12 +1,11 @@
 //! A parser for lambda expressions
 
 use term::*;
-use term::Term::*;
-use term::Notation::*;
 use self::Token::*;
 use self::CToken::*;
 use self::Error::*;
 use self::Expression::*;
+pub use term::Notation::*;
 
 /// A type to represent a parsing error.
 #[derive(Debug, PartialEq)]
@@ -41,13 +40,13 @@ fn tokenize_dbr(input: &str) -> Result<Vec<Token>, Error> {
      '\\' | 'λ' => { tokens.push(Lambda) },
             '(' => { tokens.push(Lparen) },
             ')' => { tokens.push(Rparen) },
-             x  => {
-                if let Some(n) = x.to_digit(16) {
+             _  => {
+                if let Some(n) = c.to_digit(16) {
                     tokens.push(Number(n as usize))
-                } else if x.is_whitespace() {
+                } else if c.is_whitespace() {
                     ()
                 } else {
-                    return Err(InvalidCharacter((i + 1, x)))
+                    return Err(InvalidCharacter((i + 1, c)))
                 }
             }
         }
@@ -77,11 +76,11 @@ fn tokenize_cla(input: &str) -> Result<Vec<CToken>, Error> {
             },
             '(' => { tokens.push(CLparen) },
             ')' => { tokens.push(CRparen) },
-             x  => {
-                if x.is_whitespace() {
+             _  => {
+                if c.is_whitespace() {
                     ()
-                } else if "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ".contains(x) {
-                    let mut name = x.to_string();
+                } else if "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ".contains(c) {
+                    let mut name = c.to_string();
                     while let Some(&(_, c)) = chars.peek() {
                         if c.is_whitespace() || c == '(' || c == ')' {
                             break
@@ -92,7 +91,7 @@ fn tokenize_cla(input: &str) -> Result<Vec<CToken>, Error> {
                     }
                     tokens.push(CName(name))
                 } else {
-                    return Err(InvalidCharacter((i + 1, x)))
+                    return Err(InvalidCharacter((i + 1, c)))
                 }
             }
         }
@@ -129,7 +128,7 @@ fn _convert_classic_tokens(tokens: &[CToken], stack: &mut Vec<String>, pos: &mut
             },
             CName(ref name) => {
                 let number = if stack.contains(name) {
-                    stack.iter().rev().position(|t| t == name).unwrap() + 1
+                    stack.iter().rev().position(|t| t == name).unwrap() + 1 // safe; ensured above
                 } else {
                     stack.len() + 1
                 };
@@ -192,8 +191,7 @@ fn _get_ast(tokens: &[Token], pos: &mut usize) -> Result<Expression, Error> {
 ///
 /// # Example
 /// ```
-/// use lambda_calculus::parser::parse;
-/// use lambda_calculus::term::Notation::*;
+/// use lambda_calculus::parser::*;
 /// use lambda_calculus::arithmetic::{succ, pred};
 ///
 /// assert_eq!(parse(&"λn. λf. λx. n (λg. λh. h (g f)) (λu. x) (λu. u)", Classic), Ok(pred()));
@@ -212,9 +210,7 @@ pub fn parse(input: &str, notation: Notation) -> Result<Term, Error> {
 
     let exprs = try!(if let Sequence(exprs) = ast { Ok(exprs) } else { Err(InvalidExpression) });
 
-    let mut stack = Vec::new();
-    let mut output = Vec::new();
-    let term = fold_exprs(&exprs, &mut stack, &mut output);
+    let term = fold_exprs(&exprs, &mut Vec::new(), &mut Vec::new());
 
     term
 }
@@ -229,9 +225,7 @@ fn fold_exprs(exprs: &[Expression], stack: &mut Vec<Expression>, output: &mut Ve
             Variable(i) => output.push(Var(i)),
             Abstraction => stack.push(Abstraction),
             Sequence(ref exprs) => {
-                let mut stack2 = Vec::new();
-                let mut output2 = Vec::new();
-                let subexpr = try!(fold_exprs(&exprs, &mut stack2, &mut output2));
+                let subexpr = try!(fold_exprs(&exprs, &mut Vec::new(), &mut Vec::new()));
                 output.push(subexpr)
             }
         }
@@ -251,14 +245,14 @@ fn fold_terms(mut terms: Vec<Term>) -> Result<Term, Error> {
         let fst = terms.remove(0);
         Ok( terms.into_iter().fold(fst, |acc, t| app(acc, t)) )
     } else if terms.len() == 1 {
-        Ok( terms.pop().unwrap() )
+        Ok( terms.pop().unwrap() ) // safe; ensured above
     } else {
         Err(EmptyExpression)
     }
 }
 
 #[cfg(test)]
-mod test {
+mod tests {
     use super::*;
 
     #[test]
@@ -296,7 +290,7 @@ mod test {
 
     #[test]
     fn alternative_lambda_parsing() {
-        assert_eq!(parse(&"\\\\\\2(321)", DeBruijn), parse(&"λλλ2(321)", DeBruijn))
+        assert_eq!(parse(&r#"\\\2(321)"#, DeBruijn), parse(&"λλλ2(321)", DeBruijn))
     }
 
     #[test]
@@ -325,18 +319,8 @@ mod test {
         assert_eq!(parse(&y, DeBruijn).expect("parsing Y failed!"),
             abs(
                 app(
-                    abs(
-                        app(
-                            Var(2),
-                            app(Var(1), Var(1))
-                        )
-                    ),
-                    abs(
-                        app(
-                            Var(2),
-                            app(Var(1), Var(1))
-                        )
-                    )
+                    abs(app(Var(2), app(Var(1), Var(1)))),
+                    abs(app(Var(2), app(Var(1), Var(1))))
                 )
             )
         );
