@@ -1,6 +1,7 @@
 //! [Lambda terms](https://en.wikipedia.org/wiki/Lambda_calculus#Lambda_terms)
 
-use self::Term::*;
+pub use self::Term::*;
+pub use self::Notation::*;
 use self::Error::*;
 use std::fmt;
 use std::borrow::Cow;
@@ -302,21 +303,20 @@ impl fmt::Display for Term {
     }
 }
 
-// TODO: make it safer (unwraps)
 #[doc(hidden)]
 pub fn show_precedence_cla(term: &Term, context_precedence: usize, depth: u32) -> String {
     match *term {
         Var(i) => {
             if depth >= i as u32 {
-                format!("{}", from_u32(depth + 97 - i as u32).unwrap())
+                format!("{}", from_u32(depth + 97 - i as u32).expect("error while printing term"))
             } else {
-                format!("{}", from_u32(96 + i as u32).unwrap())
+                format!("{}", from_u32(96 + i as u32).expect("error while printing term"))
             }
         },
         Abs(ref t) => {
             let ret = {
                 format!("{}{}.{}", if PRETTY_LAMBDA { 'λ' } else { '\\' },
-                    from_u32(depth + 97).unwrap(),
+                    from_u32(depth + 97).expect("error while printing term"),
                     show_precedence_cla(t, 0, depth + 1)
                 )
             };
@@ -386,7 +386,7 @@ macro_rules! app {
 }
 
 #[cfg(test)]
-mod test {
+mod tests {
     use super::{Var, PRETTY_LAMBDA};
     use super::Notation::DeBruijn;
     use parser::parse;
